#include <stdio.h>
#include <stdlib.h>

#include "hsv2rgb.c"

#define blue (2./3.)
#define pieslice (1./3.)

#define R3(x, A, B, C, D) \
    ((double)(C) + ((double)(x)-(double)(A)) * ((double)(D)-(double)(C)) \
                                             / ((double)(B)-(double)(A)))

void deflt(float *r, float *g, float *b, float in)
{
  float h, s, v;
  h = blue - in * 2 * pieslice;
  s = 0.5 + in / 2;
  v = in;
  HSVtoRGB(r, g, b, h, s, v);
}

void redonblue(float *r, float *g, float *b, float in)
{
  float h, s, v;
  h = blue - pieslice / 4 + in * (pieslice + pieslice / 4);
  s = 1;
  v = in;
  HSVtoRGB(r, g, b, h, s, v);
}

void yellowonblack(float *r, float *g, float *b, float in)
{
  float h, s, v;
  h = 0.15;
  s = 1;
  v = in;
  HSVtoRGB(r, g, b, h, s, v);
}

void blueonblack(float *r, float *g, float *b, float in)
{
  float h, s, v;
  h = blue;
  s = 1;
  v = in * 2;
  if (v > 1) {
    v = 1;
    s = 1 - (sqrt(in) - sqrt(2)/2) * 3.413;
    if (s < 0) s = 0;
    if (s > 1) s = 1;
  }
  HSVtoRGB(r, g, b, h, s, v);
}

void sunset(float *r, float *g, float *b, float in)
{
  float R, G, B;
  R = (in - 0.24) * 2.38; if (R > 1) R = 1; if (R < 0) R = 0;
  G = (in - 0.64) * 2.777; if (G > 1) G = 1; if (G < 0) G = 0;
  B = 3.6 * in;
  if (in > 0.277) B = 2 - B;
  if (B > 1) B = 1; if (B < 0) B = 0;
  *r = R;
  *g = G;
  *b = B;
}

void fruitsalad(float *r, float *g, float *b, float in)
{
  float h, s, v;
  h = blue + (pieslice / 6) - in; if (h < 0) h += 1;
  s = 1;
  v = 1;
  HSVtoRGB(r, g, b, h, s, v);
}

void highgain(float *r, float *g, float *b, float in)
{
  if (in <= 1. / 256) in = 0;
  else in = 0.1 + (powf(((in - 0.5) * 2.), 3.) + 1.) / 2.081;
  sunset(r, g, b, in);
}

FILE *startout(const char *n)
{
  FILE *f;
  f =fopen(n, "wb"); if (!f) abort();
  fprintf(f, "mapsize 4096\n");
  return f;
}

void endout(FILE *f)
{
  fprintf(f, "buildmap\n");
  fprintf(f, "dBmin -120\n");
  fprintf(f, "dBmax 0\n");
  fclose(f);
}

void go_hsv(void (*fn)(float *r, float *g, float *b, float in), const char *n)
{
  FILE *f;
  int i;
  float v;
  float r, g, b;

  f = startout(n);

  for (i = 0; i < 256; i++) {
    v = R3(i, 0, 255, 0, 1);
    fn(&r, &g, &b, v);
    fprintf(f, "colorb %g %g %g %g\n", v, r, g, b);
  }

  endout(f);
}

void wb(const char *n)
{
  FILE *f = startout(n);

  fprintf(f, "colorb 0 0 0 0\n");
  fprintf(f, "colorb 1 1 1 1\n");

  endout(f);
}

void bw(const char *n)
{
  FILE *f = startout(n);

  fprintf(f, "colorb 1 0 0 0\n");
  fprintf(f, "colorb 0 1 1 1\n");

  endout(f);
}

void banded(const char *n)
{
#define COL(c, R, G, B) \
  r = R3(R, 0, 255, 0, 1); \
  g = R3(G, 0, 255, 0, 1); \
  b = R3(B, 0, 255, 0, 1); \
  fprintf(f, "cell %d %g %g %g\n", c, r, g, b);
  float r, g, b;
  FILE *f = fopen(n, "wb"); if (!f) abort();
  fprintf(f, "mapsize 8\n");

  /* Qt::darkGreen */   COL(0, 0, 128, 0);
  /* Qt::green */       COL(1, 0, 255, 0);
  /* Qt::darkBlue */    COL(2, 0, 0, 128);
  /* Qt::blue */        COL(3, 0, 0, 255);
  /* Qt::darkYellow */  COL(4, 128, 128, 0);
  /* Qt::yellow */      COL(5, 255, 255, 0);
  /* Qt::darkRed */     COL(6, 128, 0, 0);
  /* Qt::red */         COL(7, 255, 0, 0);

  fprintf(f, "dBmin -120\n");
  fprintf(f, "dBmax 0\n");
  fclose(f);
}

void highlight(const char *n)
{
  int i;
  float r, g, b;
  FILE *f = fopen(n, "wb"); if (!f) abort();
  fprintf(f, "mapsize 100\n");

  for (i = 0; i < 99; i++) {
    COL(i, 0, 0, 128);
  }
  COL(i, 255, 255, 255);

  fprintf(f, "dBmin -120\n");
  fprintf(f, "dBmax 0\n");
  fclose(f);
}

void printer(const char *n)
{
#define COL2(c, R) \
  r = 1-R; \
  g = 1-R; \
  b = 1-R; \
  fprintf(f, "cell %d %g %g %g\n", c, r, g, b);
  float r, g, b;
  FILE *f = fopen(n, "wb"); if (!f) abort();
  fprintf(f, "mapsize 10\n");

  COL2(0, 0);
  COL2(1, 0);
  COL2(2, 0.4);
  COL2(3, 0.5);
  COL2(4, 0.6);
  COL2(5, 0.7);
  COL2(6, 0.8);
  COL2(7, 0.9);
  COL2(8, 1);
  COL2(9, 1);

  fprintf(f, "dBmin -120\n");
  fprintf(f, "dBmax 0\n");
  fclose(f);
}

int main(void)
{
  /* DefaultColours */
  go_hsv(deflt, "sonic_Default.txt");

  /* WhiteOnBlack */
  wb("sonic_White_on_Black.txt");

  /* BlackOnWhite */
  bw("sonic_Black_on_White.txt");

  /* RedOnBlue */
  go_hsv(redonblue, "sonic_Red_on_Blue.txt");

  /* YellowOnBlack */
  go_hsv(yellowonblack, "sonic_Yellow_on_Black.txt");

  /* BlueOnBlack */
  go_hsv(blueonblack, "sonic_Blue_on_Black.txt");

  /* Sunset */
  go_hsv(sunset, "sonic_Sunset.txt");

  /* FruitSalad */
  go_hsv(fruitsalad, "sonic_Fruit_Salad.txt");

  /* Banded */
  banded("sonic Banded.txt");

  /* Highlight */
  highlight("sonic_Highlight.txt");

  /* Printer */
  printer("sonic Printer.txt");

  /* HighGain */
  go_hsv(highgain, "sonic_High_Gain.txt");

  return 0;
}
