#include "fft_window.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

void fft_window_init(fft_window *w, int len, int type)
{
  w->b = malloc(sizeof(double) * len); if (!w->b) { printf("oom\n"); exit(1); }
  w->len = len;
  fft_window_set_type(w, type);
}

void fft_window_close(fft_window *w)
{
  free(w->b);
}

void fft_window_mul(fft_in *f, fft_window *w)
{
  int i;
  for (i = 0; i < w->len; i++) f[i] *= w->b[i];
}

char *fft_window_string(fft_window *w)
{
  switch (w->type) {
  case NONE: return "";
  case BARTLETT: return "/bartlett";
  case BLACKMAN: return "/blackman";
  case HAMMING: return "/hamming";
  case HANNING: return "/hanning";
  }
  return "ERROR";
}

void fft_window_set_type(fft_window *w, int type)
{
  int n;
  float M = w->len;
  w->type = type;
  switch (type) {
  case NONE:
    for (n = 0; n < w->len; n++) w->b[n] = 1;
    break;
  case BARTLETT:
    for (n = 0; n < w->len; n++)
      w->b[n] = 1 - 2 * fabsf(n - (M-1) / 2) / (M - 1);
    break;
  case BLACKMAN:
    for (n = 0; n < w->len; n++)
      w->b[n] = 0.42 - 0.5 * cos(2 * M_PI * n / (M-1))
              + 0.08 * cos(4 * M_PI * n / (M-1));
    break;
  case HAMMING:
    for (n = 0; n < w->len; n++)
      w->b[n] = 0.54 - 0.46 * cos(2 * M_PI * n / (M-1));
    break;
  case HANNING:
    for (n = 0; n < w->len; n++)
      w->b[n] = 0.5 * (1 - cos(2 * M_PI * n / (M-1)));
    break;
  }
}

void fft_window_type_from_string(fft_window *w, const char *s)
{
  int type = -1;
  if (!strcmp(s, "ba")) type = BARTLETT;
  else if (!strcmp(s, "bl")) type = BLACKMAN;
  else if (!strcmp(s, "hm")) type = HAMMING;
  else if (!strcmp(s, "hn")) type = HANNING;
  else if (!strcmp(s, "nn")) type = NONE;
  if (type == -1) { printf("unknown window type %s\n", s); return; }
  fft_window_set_type(w, type);
}
