#include "jackend.h"
#include "gui.h"
#include "commands.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

static void preload(const char *fn, mainwin *w)
{
  FILE *f;
  char l[1024];
  int i;
  f = fopen(fn, "rb");
  if (!f) { printf("error loading %s\n", fn); return; }
  while (1) {
    int c;
    i = 0;
    for (i = 0; i < 1023; i++) {
      c = fgetc(f);
      if (c == '\n' || c == EOF) break;
      /* remove leading spaces */
      if (i == 0 && isspace(c)) i--; else l[i] = c;
      /* remove comments */
      if (c == '#') { while (c != '\n' && c != EOF) c = fgetc(f); break; }
    }
    if (c != '\n' && c != EOF)
      { printf("line too long (limit 1023) in %s\n", fn); fclose(f); return; }
    l[i] = 0;
    if (i) do_command(w, l);
    if (c == EOF) break;
  }
  fclose(f);
}

int main(int n, char **v)
{
  jack_stuff jack;
  mainwin w;
  const char *f = NULL;
  int i;

  for (i = 1; i < n; i++) {
    if (!strcmp(v[i], "-h") || !strcmp(v[i], "--help")) {
usage:
      printf("usage: %s <options>\n", v[0]);
      printf("options:\n");
      printf("    -h | --help: show this help message\n");
      printf("    -f <file>: run commands in <file> at startup\n");
      return 0;
    }
    if (!strcmp(v[i], "-f")) {
      if (i == n-1) goto usage;
      if (f) goto usage;
      i++;
      f = v[i];
      continue;
    }
    goto usage;
  }

  /* we want to load all the files in the colormap selector, utf-8 sucks
   * for that; let's force iso-latin-1 which should work with every filename
   * one can imagine (I think)
   */
  setenv("G_FILENAME_ENCODING", "iso-8859-1", 1);

  gui_init();

  init_jack(&jack);
  init_mainwin(&w, &jack);

  gui_show(&w);

  if (f) preload(f, &w);

  gui_run();

  printf("\n");
  return 0;
}
